@@ -6,6 +6,22 @@
  * 
  * Copyright 2007-2008 by Garmin Ltd. or its subsidiaries
  *                2008 OpenedHand Ltd
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
  */
 
 /**
@@ -110,6 +126,19 @@ copy_error (GError **target, GError *source)
 	}
 	if (source) {
 		*target = g_error_copy (source);
+
+		/* If the error type is a D-Bus remote exception,
+		 * don't lose the "magic" sauce after the message string.
+		 * See the code in gerror_to_dbus_error_message() in dbus-glib */
+		if (source->domain == DBUS_GERROR &&
+		    source->code == DBUS_GERROR_REMOTE_EXCEPTION) {
+			int len;
+			g_free ((*target)->message);
+			len = strlen (source->message);
+			len += strlen (source->message + len + 1);
+			len += 2;
+			(*target)->message = g_memdup (source->message, len);
+		}
 	}
 }
 
@@ -122,7 +151,7 @@ gc_master_provider_get_provider (GcMasterProvider *master_provider)
 		return GEOCLUE_PROVIDER (priv->address);
 	}
 	if (priv->position) {
-		return GEOCLUE_PROVIDER (priv->address);
+		return GEOCLUE_PROVIDER (priv->position);
 	}
 	return NULL;
 }
@@ -948,6 +977,14 @@ gc_master_provider_new (const char *filename,
 	} else {
 		priv->provides = GEOCLUE_PROVIDE_NONE;
 	}
+
+    if (!connectivity && 
+         (priv->required_resources & GEOCLUE_RESOURCE_NETWORK)) {
+	    priv->provides &= ~GEOCLUE_PROVIDE_CACHEABLE_ON_CONNECTION;
+		priv->net_status = GEOCLUE_CONNECTIVITY_ONLINE;
+		priv->status = GEOCLUE_STATUS_AVAILABLE;
+        gc_master_provider_handle_status_change (provider);
+    }
 	
 	if (connectivity && 
 	    (priv->provides & GEOCLUE_PROVIDE_CACHEABLE_ON_CONNECTION)) {
@@ -1151,6 +1188,22 @@ gc_master_provider_is_good (GcMasterProvider     *provider,
 	        ((priv->required_resources & (~allowed_resources)) == 0));
 }
 
+void
+gc_master_provider_update_options (GcMasterProvider *provider)
+{
+	GeoclueProvider *geoclue;
+	GError *error = NULL;
+	
+	geoclue = gc_master_provider_get_provider (provider);
+	
+	if (!geoclue_provider_set_options (geoclue,
+	                                   geoclue_get_main_options (),
+	                                   &error)) {
+		g_warning ("Error setting provider options: %s\n", error->message);
+		g_error_free (error);
+	}
+}
+
 GeoclueStatus 
 gc_master_provider_get_status (GcMasterProvider *provider)
 {
